# Functions

In Bash, functions allow you to reuse code and improve readability. 
Below is an example how to define and use functions in Bash.

## Defining a Function

```bash
function eat_food() {
    # Commands go here
    echo "I love food"
}
```

## Calling a Function

```bash
# Call the function
eat_food
```

??? example "Click to show the solution"
    ```bash
    $ I love food
    ```

## Using Arguments in Functions

Functions can take arguments, accessed as $1, $2, $3, etc.

```bash
#!/bin/bash

# defining a function
function eat_food() {

    # get arguments
    food=$1
    # Commands go here
    echo "I love $food"
}

# calling a function
eat_food macaron
```

macaron is the argument here.

??? example "Click to show the solution"
    ```bash
    $ I love macaron
    ```

## Arguments with Default Values

It is possible to provide default value if when no argument is provided.

```bash
#!/bin/bash

# defining a function
function eat_food() {

    # get arguments
    food=${1:-"millefeuille"}
    # Commands go here
    echo "I love $food"
}

# calling a function
eat_food macaron
```

??? example "Click to show the solution"
    ```bash
    $ I love macaron
    ```

## Capturing Function Output

Bash functions cannot return strings directly, but they can:

* Return an exit status (integer) using `return`
* Output a string using `echo`, which can be captured.

### Status Code

Here an example on how to deal with status code returned from a function.
This piece of code determines whether the provided food name has an even or odd number of characters.

{% raw %}
```bash
#!/bin/bash

# defining a function
function eat_food() {

    # get arguments
    food=${1:-"millefeuille"}
    if (( ${#food} % 2 == 0 )); then # ${#food} is the length of the string
        return 0  # Success
    else
        return 1  # Failure
    fi

}

# calling a function
if eat_food $1; then
    echo "Even"
else
    echo "Odd"
fi
```
{% endraw %}

!!! note
    The reason we don’t need `[[ ... ]]` when checking a function’s exit status (return code) in an if statement is because `if` in Bash directly evaluates the exit status of a command or function.  
    The calling can be shrotened as `check_even 4 && echo "Even" || echo "Odd"`

### String

To store function output in a variable you must use the command substitution `$(...)`.


```bash
#!/bin/bash

# defining a function
function eat_food() {

    # get arguments
    food=${1:-"millefeuille"}
    # Commands go here
    echo "I love $food"
}

# calling a function and storing it in a variable
result=$(eat_food $1)

# printing the result
echo "$result"
```

## Global vs Local Variables

By default, variables inside functions are global, that means they are available from everywhere in the code. 
You can create a local variable (available only within the function) usin `local`.

```bash
#!/bin/bash

name="Global"

set_name() {
    local name="Local"
    echo "Inside function: $name"
}

set_name
echo "Outside function: $name"
```

??? example "Click to show the solution"
    ```bash
    Inside function: Local
    Outside function: Global
    ```


!!! Success "Quick recap"
    In this section we've learned:

    - how to define a function
    - how to call a function
    - how to pass arguments
    - how to return values (Status code and string)
    - how to capture function output
    - the difference between global and local variables
